package kr.co.vividnext.sodalive.mypage.alarm

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.widget.CheckBox
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.viewModels
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.databinding.ActivityAddAlarmBinding
import kr.co.vividnext.sodalive.mypage.alarm.db.Alarm
import kr.co.vividnext.sodalive.mypage.alarm.select_audio_content.AlarmSelectAudioContentActivity
import java.util.Calendar

class AddAlarmActivity : BaseActivity<ActivityAddAlarmBinding>(
    ActivityAddAlarmBinding::inflate
) {
    private val alarmViewModel: AlarmViewModel by viewModels()

    private lateinit var activityResultLauncher: ActivityResultLauncher<Intent>
    private lateinit var dayCheckBoxes: List<CheckBox>

    private var alarmId: Int = 0
    private var selectedContentId: Long = 0
    private var selectedContentTitle = ""
    private var selectedContentCreatorNickname = ""

    override fun onCreate(savedInstanceState: Bundle?) {
        activityResultLauncher = registerForActivityResult(
            ActivityResultContracts.StartActivityForResult()
        ) {
            if (it.resultCode == Activity.RESULT_OK) {
                this.selectedContentId = it.data?.getLongExtra(
                    Constants.EXTRA_AUDIO_CONTENT_ID,
                    0
                ) ?: 0

                this.selectedContentTitle = it.data?.getStringExtra(
                    Constants.EXTRA_AUDIO_CONTENT_TITLE
                ) ?: ""

                this.selectedContentCreatorNickname = it.data?.getStringExtra(
                    Constants.EXTRA_AUDIO_CONTENT_CREATOR_NICKNAME
                ) ?: ""

                binding.tvContentTitle.text = selectedContentTitle
            }
        }

        alarmId = intent.getIntExtra(Constants.EXTRA_ALARM_ID, -1)
        super.onCreate(savedInstanceState)
    }

    override fun setupView() {
        dayCheckBoxes = listOf(
            binding.chkSun,
            binding.chkMon,
            binding.chkTue,
            binding.chkWed,
            binding.chkThu,
            binding.chkFri,
            binding.chkSat
        )

        if (alarmId != -1) {
            alarmViewModel.getAlarmById(alarmId).observe(this) { alarm ->
                alarm?.let {
                    binding.etAlarmTitle.setText(it.title)

                    val calendar = Calendar.getInstance().apply { timeInMillis = it.time }
                    binding.timePicker.hour = calendar.get(Calendar.HOUR_OF_DAY)
                    binding.timePicker.minute = calendar.get(Calendar.MINUTE)
                    dayCheckBoxes.forEach { checkBox ->
                        checkBox.isChecked = it.days.contains(checkBox.text.toString())
                    }
                    binding.tvContentTitle.text = it.contentTitle

                    selectedContentId = it.contentId
                    selectedContentTitle = it.contentTitle
                    selectedContentCreatorNickname = it.contentCreatorNickname
                }
            }
        }

        binding.rlSelectAlarmContent.setOnClickListener {
            activityResultLauncher.launch(
                Intent(
                    applicationContext,
                    AlarmSelectAudioContentActivity::class.java
                )
            )
        }

        binding.tvSave.setOnClickListener { saveAlarm() }

        binding.toolbar.tvBack.text = "알람 설정"
        binding.toolbar.tvBack.setOnClickListener { finish() }
        binding.tvCancel.setOnClickListener { finish() }
    }

    private fun saveAlarm() {
        if (!validate()) return

        val hour = binding.timePicker.hour
        val minute = binding.timePicker.minute
        val alarmTitle = binding.etAlarmTitle.text.toString()
        val selectedDays = dayCheckBoxes.filter { it.isChecked }.map { it.text.toString() }

        val alarmTime = getAdjustedTimeInMillis(hour, minute, selectedDays)
        val alarm = Alarm(
            id = if (alarmId == -1) 0 else alarmId,
            title = alarmTitle,
            time = alarmTime,
            contentId = selectedContentId,
            contentTitle = selectedContentTitle,
            contentCreatorNickname = selectedContentCreatorNickname,
            days = selectedDays.toList(),
        )

        if (alarmId > 0) {
            alarmViewModel.update(alarm)
        } else {
            alarmViewModel.insert(alarm)
        }
        finish()
    }

    private fun getAdjustedTimeInMillis(hour: Int, minute: Int, selectedDays: List<String>): Long {
        val alarmTimeInMillis = getTimeInMillis(hour, minute)
        return if (selectedDays.isEmpty() && alarmTimeInMillis <= System.currentTimeMillis()) {
            getTimeInMillis(hour + 24, minute) // 다음 날로 설정
        } else {
            alarmTimeInMillis
        }
    }

    private fun getTimeInMillis(hour: Int, minute: Int): Long {
        val calendar = Calendar.getInstance()
        calendar.set(Calendar.HOUR_OF_DAY, hour)
        calendar.set(Calendar.MINUTE, minute)
        calendar.set(Calendar.SECOND, 0)
        calendar.set(Calendar.MILLISECOND, 0)
        return calendar.timeInMillis
    }

    private fun validate(): Boolean {
        if (
            selectedContentId <= 0 ||
            selectedContentTitle.isBlank() ||
            selectedContentCreatorNickname.isBlank()
        ) {
            Toast.makeText(applicationContext, "알람 콘텐츠를 선택하세요", Toast.LENGTH_LONG).show()
            return false
        }

        return true
    }
}
